{% extends 'generic/object_edit.html' %}
{% load static %}
{% load form_helpers %}
{% load helpers %}

{% block form %}
    <div class="field-group-mb-5">
        <div class="row mb-3">
            <label class="col-sm-3 col-form-label text-lg-end required">
                {{ object.assigned_object|meta:"verbose_name"|bettertitle }}</label>
            <div class="col-sm-9">
                <div class="form-control-plaintext">{{ object.assigned_object|linkify }}</div>
            </div>
        </div>
        {{ block.super }}
    </div>
    {% include 'netbox_secrets/inc/private_key_modal.html' %}
{% endblock form %}
