{% extends 'generic/object.html' %}
{% load helpers %}
{% load plugins %}

{% block breadcrumbs %}
    <li class="breadcrumb-item"><a href="{% url 'plugins:netbox_secrets:secretrole_list' %}">Secret Roles</a></li>
    <li class="breadcrumb-item">{{ object }}</li>
{% endblock %}

{% block content %}
    <div class="row mb-3">
        <div class="col col-md-6">
            <div class="card">
                <h5 class="card-header">
                    Secret Role
                </h5>
                <div class="card-body">
                    <table class="table table-hover attr-table">
                        <tr>
                            <th scope="row">Name</th>
                            <td>{{ object.name }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Description</th>
                            <td>{{ object.description|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Secrets</th>
                            <td>
                                <a href="{% url 'plugins:netbox_secrets:secret_list' %}?role_id={{ object.pk }}">{{ object.secrets.count }}</a>
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Description</th>
                            <td>{{ object.description|placeholder }}</td>
                        </tr>
                    </table>
                </div>
            </div>
            {% plugin_left_page object %}
        </div>
        <div class="col col-md-6">
            {% include 'inc/panels/tags.html' %}
            {% include 'inc/panels/custom_fields.html' %}
            {% include 'inc/panels/comments.html' %}
            {% plugin_right_page object %}
        </div>
    </div>
{% endblock %}
