{% extends "admin/base_site.html" %}

{% block content %}

<form action="." method="post">
    {% csrf_token %}
    {{ form.as_p }}
    <input type="hidden" name="action" value="activate_selected" />
    <input type="submit" name="activate" value="Activate User Key(s)" />
</form>

{% endblock %}
