# Generated by Django 4.1.5 on 2023-01-26 14:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_secrets', '0005_alter_secret_custom_field_data_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userkey',
            name='created',
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
        migrations.AlterField(
            model_name='userkey',
            name='last_updated',
            field=models.DateTimeField(auto_now=True, null=True),
        ),
    ]
