{% if perms.netbox_secrets.add_secret %}
    <a class="btn btn-primary btn-sm"
       href="{% url 'plugins:netbox_secrets:secret_add' %}?assigned_object_type={{ object|content_type_id }}&assigned_object_id={{ object.pk }}&return_url={{ request.path }}"><i
            class="mdi mdi-plus-thick"></i> Add Secret</a>
{% endif %}
